/* $Id: dynamicsign.c,v 1.1 2014/11/17 17:23:55 tm Exp $
 *
 * PDFlib PLOP DS:
 * generate a dynamic visualization document in memory with PDFlib,
 * and digitally sign a second document, using the dynamic document
 * for signature visualization.
 */

#include <stdio.h>
#include <time.h>

#include "pdflib.h"
#include "ploplib.h"

int
main(int argc, char **argv)
{
    PDF *p;
    PLOP *plop;
    int font;
    volatile const char *volatile contents = NULL;
    long size;
    time_t timer;
    struct tm ltime;
    char timebuf[256];

    const char *months[] = {
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December"
    };

    /* This is where input files live. Adjust as necessary. */
    char *searchpath = "../data";

    /* Basic signature options */
    static const char sign_opts[] =
        "sigtype=cms "      /* use sigtype=cades for PAdES signature */
        "ltv=try "          /* use ltv=full to ensure LTV status */
#if defined(USE_PKCS11)
        "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
        "engine=builtin digitalid={filename=demorsa2048.p12} password={demo} "
#endif
    ;

    if (argc < 3 )
    {
	printf("usage: dynamicsign <infile> <outfile>\n");
	return 2;
    }

    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(2);
    }

    if ((p = PDF_new()) == (PDF *) 0)
    {
        printf("Couldn't create PDFlib object (out of memory)!\n");
        return(98);
    }

    /* ---------- Step 1: create visualization document in memory with PDFlib */

    PDF_TRY(p)
    {
	char textoptlist[1024];

	/* create a new PDF document in memory */

	PDF_begin_document(p, "", 0, "");

	PDF_set_info(p, "Creator", "dynamicsign.c");
	PDF_set_info(p, "Author", "Thomas Merz");
	PDF_set_info(p, "Title", "Dynamic signature visualization document");

	font = PDF_load_font(p, "Helvetica", 0, "host", "");

	PDF_begin_page_ext(p, 300, 50, "");

	time(&timer);
	ltime = *localtime(&timer);

	/* Prepare dynamic text for the signature visualization */
	sprintf(timebuf, "Digitally signed %s %d, %d at %02d:%02d:%02d",
	    months[ltime.tm_mon], ltime.tm_mday, ltime.tm_year + 1900,
	    ltime.tm_hour, ltime.tm_min, ltime.tm_sec);

	sprintf(textoptlist,
	    "fontsize=10 fontname=Helvetica "
	    "encoding=unicode textformat=utf8 fillcolor=red");
	PDF_fit_textline(p, timebuf, 0, 10, 10, textoptlist);
		
	PDF_end_page_ext(p, "");

	PDF_end_document(p, "");

	/* fetch the memory buffer containing the PDF document */
	contents = PDF_get_buffer(p, &size);
    }

    PDF_CATCH(p)
    {
        printf("PDFlib exception occurred in dynamicsign sample:\n");
        printf("[%d] %s: %s\n",
            PDF_get_errnum(p), PDF_get_apiname(p), PDF_get_errmsg(p));
        PDF_delete(p);
        return(98);
    }

    /* ------------------ Step 2: sign the document ----------------- */

    /* create a new PLOP object */
    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(99);
    }

    PLOP_TRY(plop)
    {
	int doc, visdoc;
	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	/* Create a virtual PVF file from the document data in memory */
	PLOP_create_pvf(plop, "/pvf/visdoc.pdf", 0,
		(const char *) contents, size, "");

	if ((doc = PLOP_open_document(plop, argv[1], 0, "")) == -1)
	{
	    printf("Error: %s: %s\n", PLOP_get_apiname(plop),
	                                PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	/* open visualization document from PVF file */
	if ((visdoc = PLOP_open_document(plop, "/pvf/visdoc.pdf", 0, "")) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}

	/* Add field option for visualization to base signature options */
	sprintf(optlist, "%s field={visdoc=%d rect={10 10 adapt adapt}} ",
	    	sign_opts, visdoc);

	if (PLOP_prepare_signature(plop, optlist) == -1)
	{
	    printf("Error: %s: %s\n",
		    PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	/* sign the output file */
	sprintf(optlist, "input=%d", doc);
	if (PLOP_create_document(plop, argv[2], 0, optlist) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}

	/* close the input document */
        PLOP_close_document(plop, visdoc, "");

	/* delete PVF file containing input document */
	if (PLOP_delete_pvf(plop, "/pvf/visdoc.pdf", 0) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}
    }

    PLOP_CATCH(plop)
    {
	printf("PLOP exception occurred in dynamicsign:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(99);
    }

    /* delete the PLOP object */
    PLOP_delete(plop);

    /* delete the PDFlib object */
    PDF_delete(p);

    return 0;
}
